@@ -33,35 +33,35 @@ from mesonbuild import mesonmain
 from mesonbuild import mtest
 from mesonbuild import mlog
 from mesonbuild.environment import Environment, detect_ninja, detect_machine_info
-from mesonbuild.coredata import backendlist, version as meson_version
-from mesonbuild.mesonlib import OptionKey, setup_vsenv
+from mesonbuild.coredata import version as meson_version
+from mesonbuild.options import backendlist
+from mesonbuild.mesonlib import setup_vsenv
+from mesonbuild.options import OptionKey
 
 if T.TYPE_CHECKING:
     from mesonbuild.coredata import SharedCMDOptions
 
-NINJA_1_9_OR_NEWER = False
+NINJA_1_12_OR_NEWER = False
 NINJA_CMD = None
 # If we're on CI, detecting ninja for every subprocess unit test that we run is slow
 # Optimize this by respecting $NINJA and skipping detection, then exporting it on
 # first run.
 try:
-    NINJA_1_9_OR_NEWER = bool(int(os.environ['NINJA_1_9_OR_NEWER']))
+    NINJA_1_12_OR_NEWER = bool(int(os.environ['NINJA_1_12_OR_NEWER']))
     NINJA_CMD = [os.environ['NINJA']]
 except (KeyError, ValueError):
-    # Look for 1.9 to see if https://github.com/ninja-build/ninja/issues/1219
-    # is fixed
-    NINJA_CMD = detect_ninja('1.9')
+    # Look for 1.12, which removes -w dupbuild=err
+    NINJA_CMD = detect_ninja('1.12')
     if NINJA_CMD is not None:
-        NINJA_1_9_OR_NEWER = True
+        NINJA_1_12_OR_NEWER = True
     else:
-        mlog.warning('Found ninja <1.9, tests will run slower', once=True)
         NINJA_CMD = detect_ninja()
 
 if NINJA_CMD is not None:
-    os.environ['NINJA_1_9_OR_NEWER'] = str(int(NINJA_1_9_OR_NEWER))
+    os.environ['NINJA_1_12_OR_NEWER'] = str(int(NINJA_1_12_OR_NEWER))
     os.environ['NINJA'] = NINJA_CMD[0]
 else:
-    raise RuntimeError('Could not find Ninja v1.7 or newer')
+    raise RuntimeError('Could not find Ninja.')
 
 # Emulate running meson with -X utf8 by making sure all open() calls have a
 # sane encoding. This should be a python default, but PEP 540 considered it not
@@ -135,10 +135,6 @@ class FakeBuild:
     def __init__(self, env):
         self.environment = env
 
-class FakeCompilerOptions:
-    def __init__(self):
-        self.value = []
-
 def get_fake_options(prefix: str = '') -> SharedCMDOptions:
     opts = T.cast('SharedCMDOptions', argparse.Namespace())
     opts.native_file = []
@@ -148,11 +144,11 @@ def get_fake_options(prefix: str = '') -> SharedCMDOptions:
     opts.cmd_line_options = {}
     return opts
 
-def get_fake_env(sdir='', bdir=None, prefix='', opts=None):
+def get_fake_env(sdir: str = '', bdir: T.Optional[str] = None, prefix: str = '',
+                 opts: T.Optional[SharedCMDOptions] = None) -> Environment:
     if opts is None:
         opts = get_fake_options(prefix)
     env = Environment(sdir, bdir, opts)
-    env.coredata.options[OptionKey('args', lang='c')] = FakeCompilerOptions()
     env.machines.host.cpu_family = 'x86_64' # Used on macOS inside find_library
     # Invalidate cache when using a different Environment object.
     clear_meson_configure_class_caches()
@@ -271,7 +267,9 @@ def get_backend_commands(backend: Backend, debug: bool = False) -> \
         test_cmd = cmd + ['-target', 'RUN_TESTS']
     elif backend is Backend.ninja:
         global NINJA_CMD
-        cmd = NINJA_CMD + ['-w', 'dupbuild=err', '-d', 'explain']
+        cmd = NINJA_CMD + ['-d', 'explain']
+        if not NINJA_1_12_OR_NEWER:
+            cmd += ['-w', 'dupbuild=err']
         if debug:
             cmd += ['-v']
         clean_cmd = cmd + ['clean']
@@ -282,22 +280,7 @@ def get_backend_commands(backend: Backend, debug: bool = False) -> \
         raise AssertionError(f'Unknown backend: {backend!r}')
     return cmd, clean_cmd, test_cmd, install_cmd, uninstall_cmd
 
-def ensure_backend_detects_changes(backend: Backend) -> None:
-    global NINJA_1_9_OR_NEWER
-    if backend is not Backend.ninja:
-        return
-    need_workaround = False
-    # We're using ninja >= 1.9 which has QuLogic's patch for sub-1s resolution
-    # timestamps
-    if not NINJA_1_9_OR_NEWER:
-        mlog.warning('Don\'t have ninja >= 1.9, enabling timestamp resolution workaround', once=True)
-        need_workaround = True
-    # Increase the difference between build.ninja's timestamp and the timestamp
-    # of whatever you changed: https://github.com/ninja-build/ninja/issues/371
-    if need_workaround:
-        time.sleep(1)
-
-def run_mtest_inprocess(commandlist: T.List[str]) -> T.Tuple[int, str, str]:
+def run_mtest_inprocess(commandlist: T.List[str]) -> T.Tuple[int, str]:
     out = StringIO()
     with mock.patch.object(sys, 'stdout', out), mock.patch.object(sys, 'stderr', out):
         returncode = mtest.run_with_args(commandlist)
@@ -397,7 +380,7 @@ def main():
         else:
             print(mlog.bold('Running unittests.'))
             print(flush=True)
-            cmd = mesonlib.python_command + ['run_unittests.py', '-v'] + backend_flags
+            cmd = mesonlib.python_command + ['run_unittests.py'] + backend_flags
             if options.failfast:
                 cmd += ['--failfast']
             returncode += subprocess_call(cmd, env=env)
